<?php
/*
Plugin Name: PictPress
Plugin URI: http://www.curioso.org/pictpress/
Description: Adds fotolog functionality to wordpress posts by adding thumbnails and picture pages automatically to the post for all pictures stored at the post's permalink location.
Version: 0.91
Author: Curioso
Author URI: http://www.curioso.org/
*/ 

//-------------------------------------------------------------------
// Configuration parameters
// Edit these to suit your needs
//-------------------------------------------------------------------

// Location of image directory relative to the root of your site.
// The same variables can be used as in permalink specifications:
//         %year%
//         %monthnum%
//         %day%
//         %postname%
//         %post_id%'
//
$pp_imagedir = "/images/%year%/%monthnum%/%postname%";

// Max horizontal/vertical size for thumbnails
//
$pp_thumb_size = 100;

// Max horizontal/vertical size for images on picture pages
//
$pp_image_size = 600;

// Max number of thumbnails per page
// Set to 0 for no limit
//
$pp_max_thumbs = 0;

// Number of thumnails before more
// Insert <!--more--> after so many thumbnails
// Set to 0 for no <!--more-->
//
$pp_more_thumbs = 5;

// Add header for /pictpress/pictpress.css
//
$pp_use_css = 1;

//-------------------------------------------------------------------
// End of configuration parameters
//-------------------------------------------------------------------

// Get image directory
//
function pp_get_image_path() {
	global $wpdb, $tableposts, $post_ID, $pp_imagedir;
        $post = $wpdb->get_row("SELECT post_date, post_name 
                                FROM $tableposts WHERE ID = $post_ID");

        $rewritecode = array(
                '%year%',
                '%monthnum%',
                '%day%',
                '%postname%',
                '%post_id%'
        );
	$unixtime = strtotime($post->post_date);
        $rewritereplace = array(
                date('Y', $unixtime),
                date('m', $unixtime),
                date('d', $unixtime),
                $post->post_name,
                $post_ID
        );
	$perm = str_replace($rewritecode, $rewritereplace, $pp_imagedir); 
	return $perm;
}

function pp_add_picture_pages() {
	include_once (ABSPATH . "../pictpress/class.image.php");
	include_once (ABSPATH . "../pictpress/class.imagedir.php");

        global $pp_thumb_size, $pp_image_size, $pp_max_thumbs, $pp_more_thumbs,
	       $content, $wpdb, $tableposts, $post_ID;

	$path = pp_get_image_path();
	$dirname = ABSPATH . ".." . $path;

        if (!file_exists($dirname) || !is_dir($dirname)) return;

	$dir = new ImageDir($dirname);
	if ($dir->count == 0) return;

	// Patterns for thumbnails and picture pages
	//
	$thumbspat = "/<!--pp-thumb-start-->.*?<!--pp-thumb-end-->/s";
	$pictpat = "/<!--pp-page-start-->.*<!--pp-page-end-->/s";

	// Remove previous thumbnails
	//
	$content = preg_replace($thumbspat, "", $content);

	$pages = explode('<!--nextpage-->', $content);
	$numpages = count($pages);

	// Copy all pages that contain no PictPress images
	//
	for ($p = 0; $p < $numpages; $p++) {
		if (!preg_match($pictpat, $pages[$p])) {
			$newpages[] = $pages[$p];
		}
	}

	// Determine number of first page with PictPress images
	//
	if ($pp_max_thumbs) {
		$first = count($newpages) + ceil($dir->count / $pp_max_thumbs);
	} else {
		$first = count($newpages) + 1;
	}

	// Create thumbnails
	//
	$clear = "<div class=\"clear\">&nbsp;</div>\n";
	$thumbs = "<!--pp-thumb-start-->$clear";
	for ($i = 0; $i < $dir->count; $i++) {
		if ($pp_more_thumbs && $i == $pp_more_thumbs) {
			$thumbs .= "<!--more-->";
		}
		if ($i && $pp_max_thumbs && ($i % $pp_max_thumbs) == 0) {
			$thumbs .= "$clear<!--nextpage-->$clear";
		}
                $p = $first + $i;
		$image = $dir->GetImage($i);
		$thumbs .= "<div class=\"thumbnail\">\n";
		$thumbs .= "<div class=\"image\">\n";
		$thumbs .= $image->GetThumbnail($path, pp_get_permalink($p), $pp_thumb_size);
		$thumbs .= "</div>\n";
		$thumbs .= "<div class=\"caption\">";
		$thumbs .= "<!--subtitle $p-->";
		$thumbs .= "</div>\n";
		$thumbs .= "</div>\n";
	}
	$thumbs .= "$clear<!--pp-thumb-end-->";

	// Append thumbnails to end of first page.
	//
	$newpages[0] .= $thumbs;

	// Add one page per image to end of post
	//
	for ($i = 0; $i < $dir->count; $i++) {
		$image = $dir->GetImage($i);
		$name = basename($image->name);

		// Check if there is an existing page with this image
		//
		$oldpage = '';
		for ($p = 1; $p < $numpages; $p++) {
			if (preg_match("/<!--pp-page-start-->.*$name.*<!--pp-page-end-->/s", $pages[$p])) {
				$oldpage = $pages[$p];
				break;
			}
		}
		// If not, generate a page with caption and scaled image
		//
		if (!$oldpage) {
			$oldpage .= "<!--pp-page-start-->\n";
			$oldpage .= "<h4 class=\"caption\">";
			$oldpage .= $image->GetCleanComment();
			$oldpage .= "</h4>\n";
			$oldpage .= "<div class=\"image\">\n";
			$oldpage .= $image->GetThumbnail($path, "$path/$name", $pp_image_size);
			$oldpage .= "</div>\n";
			$oldpage .= "<!--pp-page-end-->\n";
		}
		$newpages[] = $oldpage;
	}

	// At this point we have lost all pages for which the associated image
	// has disappeared.

	$content = implode("<!--nextpage-->", $newpages);

        // Update database
        $result = $wpdb->query("
                UPDATE $tableposts SET
                        post_content = '$content'
                WHERE ID = $post_ID ");
}

// Add CSS header (to be called from wp_head)
//
function pp_css_head () {
	echo "<style type=\"text/css\">";
	echo "@import url( /pictpress/pictpress.css );";
        echo "</style>";
}

// Get subtitle for page $p
//
function pp_get_subtitle ($p) {
	global $pages;

	if (preg_match("'<h4.*?>(.*?)</h4>'s", $pages[$p-1], $matches)) {
		return $matches[1];
	} else {
		return "Page $p";
	}
}

// Get permalink for page $p
//
function pp_get_permalink ($p) {
	global $querystring_equal, $querystring_separator, $post_ID;

	if ('' == get_settings('permalink_structure')) {
		return get_permalink($post_ID).$querystring_separator.'page'.$querystring_equal.$p;
	} else {
		return get_permalink($post_ID).$p.'/';
	}

}

// Expand subtitles in content (to be called as filter in the_content)
//
function pp_expand_subtitles ($content) {
	return preg_replace_callback("/<!--subtitle +([0-9]+)-->/", 
			create_function('$matches',
				'return pp_get_subtitle($matches[1]);'),
			$content);
}

add_action("publish_post", "pp_add_picture_pages");
add_filter("the_content", "pp_expand_subtitles");
add_filter("the_excerpt", "pp_expand_subtitles");
if ($pp_use_css) add_action("wp_head", "pp_css_head");

?>
